from django.views.generic.dates import DateDetailView, YearArchiveView, \
    ArchiveIndexView, DayArchiveView, WeekArchiveView, MonthArchiveView, \
    TodayArchiveView
from django.views.generic.detail import DetailView
from django.views.generic.list import ListView


class AjaxResponseMixin(object):
    fragment_name = None
    fragment_suffix = '_fragment'

    def get_template_names(self):
        if self.request.is_ajax():
            if self.fragment_name is not None:
                self.template_name = self.fragment_name
            self.template_name_suffix += self.fragment_suffix

        if hasattr(self.parent, "get_template_names"):
            return super(self.parent, self).get_template_names()

        return [self.template_name, "%s%s"
                % (self.template_name, self.template_name_suffix)]

views = (DetailView, ListView, DateDetailView, TodayArchiveView,
         DayArchiveView, WeekArchiveView, MonthArchiveView, YearArchiveView,
         ArchiveIndexView)


# Inject AjaxResponseMixin into each view and tack "Octopus" on to the
# beginning of the class .
for view in views:
    name = 'Octopus%s' % view.__name__

    locals()[name] = type(name, (AjaxResponseMixin, view), {
        'parent': view
    })


from django.views.generic.dates import DayArchiveView
from octopus.views import OctopusDetailView, OctopusListView, \
    OctopusDateDetailView, OctopusYearArchiveView, OctopusArchiveIndexView, \
    OctopusDayArchiveView, OctopusWeekArchiveView, OctopusMonthArchiveView, \
    OctopusTodayArchiveView
from test_app.models import TestModel


class DetailView(OctopusDetailView):
    model = TestModel
    template_name = "full.html"
    fragment_name = "fragment.html"


class ListView(OctopusListView):
    model = TestModel
    template_name = "full.html"
    fragment_name = "fragment_list.html"


class SuffixView(OctopusDetailView):
    model = TestModel


class DateDetailView(OctopusDateDetailView):
    model = TestModel
    date_field = 'date'


class YearArchiveView(OctopusYearArchiveView):
    model = TestModel
    date_field = 'date'


class ArchiveIndexView(OctopusArchiveIndexView):
    model = TestModel
    date_field = 'date'


class DayArchiveView(OctopusDayArchiveView):
    model = TestModel
    date_field = 'date'


class WeekArchiveView(OctopusWeekArchiveView):
    model = TestModel
    date_field = 'date'
    week_format = "%W"


class MonthArchiveView(OctopusMonthArchiveView):
    model = TestModel
    date_field = 'date'


class TodayArchiveView(OctopusTodayArchiveView):
    model = TestModel
    date_field = 'date'



from datetime import date
from django.test import TestCase, RequestFactory, Client
from test_app.models import TestModel


class ViewsTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.m = TestModel()
        self.m.date = date(2015, 2, 21)
        self.id = self.m.save()
        self.n = TestModel()
        self.n.date = date.today()
        self.n.save()

    def test_ajax_DetailView(self):

        response = self.client.get('/detail/%d' % self.m.id,  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest")
        self.assertTrue("fragment.html" in response.template_name)

    def test_full_DetailView(self):
        response = self.client.get('/detail/%d' % self.m.id)

        self.assertTrue("full.html" in response.template_name)

    def test_ajax_ListView(self):
        response = self.client.get(
            '/list/',
            {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest")

        self.assertTrue("fragment_list.html" in response.template_name)

    def test_full_ListView(self):
        response = self.client.get('/list/')

        self.assertTrue("full.html" in response.template_name)

    def test_ajax_SuffixView(self):
        response = self.client.get('/suffix/%d' % self.m.id,  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest")
        self.assertEqual(b"suffix", response.content)

    def test_ajax_OctopusArchiveIndexView(self):
        response = self.client.get('%s' % 'archive',  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest")
        self.assertEqual(b"archive", response.content)

    def test_ajax_OctopusDateDetailView(self):
        response = self.client.get('/2015/feb/21/%d/' % self.m.id,  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest", follow=True)
        self.assertEqual(b"suffix", response.content)


    def test_ajax_OctopusYearArchiveView(self):
        response = self.client.get('/2015/',  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest", follow=True)
        self.assertEqual(
            [u'test_app/testmodel_archive_year_fragment.html'],
            response.template_name)


    def test_ajax_OctopusArchiveIndexView(self):
        response = self.client.get('/archive/',  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest", follow=True)
        self.assertEqual(
            [u'test_app/testmodel_archive_fragment.html'],
            response.template_name)

    def test_ajax_OctopusDayArchiveView(self):
        response = self.client.get('/2015/feb/21/',  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest", follow=True)
        self.assertEqual(
            [u'test_app/testmodel_archive_day_fragment.html'],
            response.template_name)

    def test_ajax_OctopusWeekArchiveView(self):
        response = self.client.get('/2015/week/07/',  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest", follow=True)
        self.assertEqual(
            [u'test_app/testmodel_archive_week_fragment.html'],
            response.template_name)

    def test_ajax_OctopusMonthArchiveView(self):
        response = self.client.get('/2015/feb/',  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest", follow=True)
        self.assertEqual(
            [u'test_app/testmodel_archive_month_fragment.html'],
            response.template_name)

    def test_ajax_OctopusTodayArchiveView(self):
       response = self.client.get('/today/',  {},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest", follow=True)
       self.assertEqual(
           [u'test_app/testmodel_archive_day_fragment.html'],
           response.template_name)



from django.conf import settings
from django.conf.urls import patterns, url
from django.views.generic.base import TemplateView
from test_app.views import DetailView, ListView, SuffixView, \
    DateDetailView, YearArchiveView, \
    ArchiveIndexView, DayArchiveView, WeekArchiveView, MonthArchiveView, \
    TodayArchiveView


urlpatterns = patterns(
   '',
   url(r'^/?$', TemplateView.as_view(template_name='base.html'), name="home"),
   url(r'^list/', ListView.as_view(), name="list"),
   url(r'^detail/(?P<pk>\d)', DetailView.as_view(), name="detail"),
   url(r'^suffix/(?P<pk>\d)', SuffixView.as_view(), name="suffix"),
   url(r'^multi/(?P<pk>\d)/(?P<s>\w)', DetailView.as_view(), name="multi"),
   url(r'^(?P<year>\d{4})/week/(?P<week>\d+)/$', WeekArchiveView.as_view(),
     name='week'),
   url(r'^(?P<year>\d{4})/(?P<month>[-\w]+)/(?P<day>\d+)/$',
       DayArchiveView.as_view(), name='day'),
   url(r'^(?P<year>\d+)/(?P<month>[-\w]+)/(?P<day>\d+)/(?P<pk>\d+)/$',
        DateDetailView.as_view(), name="date"),
   url(r'^archive/$', ArchiveIndexView.as_view(), name="archive"),
   url(r'^(?P<year>\d{4})/$', YearArchiveView.as_view(), name='year'),

   url(r'^(?P<year>\d{4})/(?P<month>[-\w]+)/$', MonthArchiveView.as_view(),
     name='month'),
   url(r'^today/$', TodayArchiveView.as_view(), name='today'),
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.conf import settings


TRUE_SETTINGS = [
    'ALLOW_MANUAL',     # Allows manual paths and potentially external URLS
    'FAIL_SILENTLY',    # Currently not being used
]

# FALSE_SETTINGS = []

for setting in TRUE_SETTINGS:
    locals()[setting] = getattr(settings, "OCTOPUS_%s" % setting, True)



import os

INSTALLED_APPS = [
    'octopus',
    'test_app'
]

SECRET_KEY = '1'

DEBUG = True

STATIC_URL = '/static/'
MEDIA_URL = '/media/'

APPEND_SLASHES = True

TEMPLATE_DIRECTORIES = (
    os.path.join(os.path.dirname(os.path.realpath(__file__)),
                 'test_app/templates'))

ROOT_URLCONF = "test_app.urls"

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.db',
    }
}



from collections import OrderedDict
from django.template.loader import render_to_string
from django.test import TestCase, RequestFactory, Client
from octopus.templatetags.a import a
from test_app.models import TestModel


class TagsTest(TestCase):
    def setUp(self):
        self.m = TestModel()
        self.m.save()
        self.id = self.m.id

    def test_a_single_arg(self):
        self.assertEqual(
            a(
                "text",
                "target",
                "detail",
                self.id,
                action="append",
                classes="poop",
                id="man",
                title="title"),
            OrderedDict({
                'id': 'man',
                'target': 'target',
                'action': 'append',
                'method': 'get',
                'classes': 'poop',
                'href': '/detail/1',
                'title': 'title',
                'text': 'text'
            })
        )

    def test_a_multi_arg(self):
        self.assertEqual(
            a(
                "text",
                "target",

                "multi",
                self.id,
                'a',
                action="prepend",
                method='get',
                classes="poop",
                id="man",
                title="title"),
            OrderedDict({
                'id': 'man',
                'target': 'target',
                'action': 'prepend',
                'classes': 'poop',
                'method': 'get',
                'href': '/multi/1/a',
                'title': 'title',
                'text': 'text'
            })
        )
    def test_a_no_arg(self):
        self.assertEqual(
            a(
                "text",
                "target",
                "list",
                method='post',
                classes="poop",
                id="man",
                title="title"),
            OrderedDict({
                'id': 'man',
                'target': 'target',
                'action': 'replace',
                'method':'post',
                'classes': 'poop',
                'href': '/list/',
                'title': "title",
                'text':'text'
            })
        )

    def test_render_template(self):
        context = a(
            "text",
            "target",
            "detail",
            self.id,
            action="append",
            classes="poop",
            id="man",
            title="title"
        )

        self.assertEqual(render_to_string('octopus/link.html', context),
            u'<a id="man" target="target" action="append" method="get" class="octopus-link poop" href="/detail/%d" title="title">text</a>' % self.id
        )

    def test_render_template_nokwargs(self):
        context = a(
            "text",
            "target",
            "detail",
            self.id,
            action="prepend",
         )

        self.assertEqual(render_to_string('octopus/link.html', context),
            u'<a target="target" action="prepend" method="get" class="octopus-link" href="/detail/%d" title="None">text</a>' % self.id
        )


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Copyright (c) 2014 Brian McClure
#
#  django-octopus is free software under terms of the MIT License.
#

from setuptools import setup, find_packages

description = """"""

setup(
    name     = 'django-octopus',
    version  = '0.1',
    packages = find_packages(),
    include_package_data=True,
    requires = ['python (>= 2.7)', 'django (>= 1.6)'],
    description  = 'A simple pull framework for django',
    author       = 'Brian McClure',
    author_email = 'brian.mcclr@gmail.com',
    url          = 'https://github.com/brmc/django-octopus',
    download_url = 'https://github.com/brmc/django-octopus.git',
    license      = 'MIT License',
    keywords     = 'django, ajax, front-end, pull',
    classifiers  = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from collections import OrderedDict
from django import template
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse, NoReverseMatch
from octopus import settings

register = template.Library()


@register.inclusion_tag('octopus/link.html')
def a(text, target, url_name, *url_args, **kwargs):
    """
    builds a link that can be used by static/octopus.js.


    Arguments:
    :param text: The link text: </a>Text</a>
    :type text: str
    :param target: The html element, class, or id to catch the returned data
    :type target: str with the selector prefix if applicable: body, .className,
        #idname
    :param action: Dictates how to handle the incoming data. Allowed values:
        replace, append, prepend.
        Defaults to replace
    :type action: str
    :param url_name: The name of the url pattern defined in your urls.py
    :type url_name: str
    :param url_args: Any arguments to be passed to this url
    :param kwargs: Expected kwargs(with default in parentheses):
        method("get"), action("replace"), classes(None), id(None), title(None)
    :type kwargs: str
    :return: OrderedDict
    """

    try:
        href = reverse(url_name, args=url_args)
    except NoReverseMatch:
        if settings.ALLOW_MANUAL is True:
            href = url_name
        else:
            raise NoReverseMatch

    # I'm setting default values here rather than above so that the tag would
    # be easier to use and feel more natural.
    method, action, classes, id_, title = \
        map(kwargs.get, ['method', 'action', 'classes', 'id', 'title'],
                        ['get',    'replace', None,     None,  None, ])

    if action.lower() not in ['replace', 'append', 'prepend']:
        raise ImproperlyConfigured("%s is not a valid value for action. It "
                                   "should be: replace, append, or prepend.")

    return OrderedDict({
        'id': id_,
        'target': target,
        'action': action,
        'classes': classes,
        'method': method,
        'href': href,
        'title': title,
        'text': text
    })
